{% extends "base.html" %}

{% block content %}
    <h1>Pocket Settings</h1>
	
	<p>You need to register a pocket API key to recover your account's bookmarks and then continuously sync the bookmarks with pocket. You can do that <a href="https://getpocket.com/developer/apps/new" rel="noopener">here.</a> It only needs the <code>Retrieve</code> permission and the platform is <code>Web</code>. Enter your consumer key below, or if you've already done this, <a href="/parse_pocket">go here</a>.

	<b>It can take a while if you have many bookmarks.</b>
    <form action="" method="post" novalidate>
        {{ form.hidden_tag() }}
        {% for error in form.api_key.errors %}
            <span style="color: red;">{{ error }}</span>
        {% endfor %}
        <p>
            {{ form.api_key.label }}
            {{ form.api_key }}
        <p>{{ form.submit() }}</p>
    </form>
{% endblock %}
